@@ -85,10 +85,12 @@ static void append_query_plan(ExplainState *es);
 /* on_shmem_exit() callback to delete hash entry on client disconnect. */
 static void cleanup(int code, Datum arg);
 /* Set extension state, either enable or disable. */
-static void set_state(const bool state);
+static void set_state(bool state, void *extra);
 static const char *show_state(void);
 /* Set query plan output format: text, json, ... */
 static void set_format(const int format);
+/* Propagate GUC variable value to shared memory (assign hook). */
+static void prop_format_to_shmem(int newval, void *extra);
 static const char *show_format(void);
 /* Check the extension has been properly loaded. */
 static inline void shmem_safety_check(void);
@@ -106,23 +108,9 @@ static void pgsp_ExecutorStart(QueryDesc *queryDesc, int eflags);
 static void pgsp_ExecutorRun(QueryDesc *queryDesc, ScanDirection direction,
                              uint64 count, bool execute_once);
 
-/* Enables/Disables the extension. */
-Datum pg_show_plans_enable(PG_FUNCTION_ARGS);
-Datum pg_show_plans_disable(PG_FUNCTION_ARGS);
-/* Sets query plan output format. */
-Datum pgsp_format_text(PG_FUNCTION_ARGS);
-Datum pgsp_format_json(PG_FUNCTION_ARGS);
-Datum pgsp_format_yaml(PG_FUNCTION_ARGS);
-Datum pgsp_format_xml(PG_FUNCTION_ARGS);
 /* Show query plans of all the currently running statements. */
 Datum pg_show_plans(PG_FUNCTION_ARGS);
 
-PG_FUNCTION_INFO_V1(pg_show_plans_enable);
-PG_FUNCTION_INFO_V1(pg_show_plans_disable);
-PG_FUNCTION_INFO_V1(pgsp_format_text);
-PG_FUNCTION_INFO_V1(pgsp_format_json);
-PG_FUNCTION_INFO_V1(pgsp_format_yaml);
-PG_FUNCTION_INFO_V1(pgsp_format_xml);
 PG_FUNCTION_INFO_V1(pg_show_plans);
 
 /* Global Variables */
@@ -171,9 +159,9 @@ _PG_init(void)
 	                         NULL,
 	                         &start_enabled,
 	                         true,
-	                         PGC_POSTMASTER,
+	                         PGC_USERSET,
 	                         0,
-	                         NULL, NULL, show_state);
+	                         NULL, set_state, show_state);
 	DefineCustomIntVariable("pg_show_plans.max_plan_length",
 	                        gettext_noop("Set the maximum plan length. "
 	                                     "Note that this module allocates (max_plan_length*max_connections) "
@@ -195,9 +183,9 @@ _PG_init(void)
 	                         &plan_format,
 	                         EXPLAIN_FORMAT_TEXT,
 	                         plan_formats,
-	                         PGC_POSTMASTER,
+	                         PGC_USERSET,
 	                         0,
-	                         NULL, NULL, show_format);
+	                         NULL, prop_format_to_shmem, show_format);
 
 	/* Save old hooks, and install new ones. */
 #if PG_VERSION_NUM >= 150000
@@ -217,13 +205,13 @@ _PG_init(void)
 	ExecutorRun_hook = pgsp_ExecutorRun;     /* Track nest level. */
 }
 
-Size
+static Size
 hash_entry_size(void)
 {	/* Structure size & variable array maximal length. */
 	return offsetof(pgspEntry, plan) + max_plan_length;
 }
 
-Size
+static Size
 shmem_required(void)
 {
 	Size s;
@@ -233,14 +221,14 @@ shmem_required(void)
 	return s;
 }
 
-uint32
+static uint32
 gen_hash_key(const void *key, Size keysize)
 {
 	const pgspHashKey *k = (const pgspHashKey *) key;
 	return (uint32) k->pid;
 }
 
-int
+static int
 compare_hash_key(const void *key1, const void *key2, Size keysize)
 {
 	const pgspHashKey *k1 = (const pgspHashKey *) key1;
@@ -248,7 +236,7 @@ compare_hash_key(const void *key1, const void *key2, Size keysize)
 	return (k1->pid == k2->pid) ? 0 : 1;
 }
 
-int
+static int
 ensure_cached(void)
 {
 	pgspHashKey pgvp_hash_hey;
@@ -269,7 +257,7 @@ ensure_cached(void)
 }
 
 /* Returns NULL if the hash table is full. */
-pgspEntry *
+static pgspEntry *
 create_hash_entry(const pgspHashKey *key)
 {
 	pgspEntry *entry;
@@ -282,7 +270,7 @@ create_hash_entry(const pgspHashKey *key)
 	return entry;
 }
 
-void
+static void
 append_query_plan(ExplainState *es)
 {
 	const StringInfo new_plan = es->str;
@@ -292,16 +280,13 @@ append_query_plan(ExplainState *es)
 
 	offset = 0;
 	for (i = 0; i < nest_level; i++)
-		offset += pgsp_cache->plan_len[i];
-	if (nest_level >= 1)
-		offset++; /* Point right after the '\0'. */
+		offset += pgsp_cache->plan_len[i] + 1;
 	space_left = max_plan_length - offset;
 
 	if (pgsp->plan_format == EXPLAIN_FORMAT_TEXT)
 		new_plan->len--; /* Discard '\n'. */
 
 	if (new_plan->len < space_left) { /* Enough space for a new plan. */
-		/* -1 to dispose of '\n'. */
 		memcpy(pgsp_cache->plan + offset,
 		       new_plan->data, new_plan->len);
 		pgsp_cache->plan[offset + new_plan->len] = '\0';
@@ -317,7 +302,8 @@ append_query_plan(ExplainState *es)
 	pgsp_cache->n_plans = nest_level+1;
 }
 
-void cleanup(int code, Datum arg)
+static void
+cleanup(int code, Datum arg)
 {
 	pgspHashKey key;
 	key.pid = pgsp_cache->hash_key.pid;
@@ -326,11 +312,15 @@ void cleanup(int code, Datum arg)
 	LWLockRelease(pgsp->lock);
 }
 
-void
-set_state(const bool state)
+static void
+set_state(bool state, void *extra)
 {
-	shmem_safety_check();
-	if (is_allowed_role())
+	/* Shared memory may not be fully available at server start, so we do not
+	 * check for pgsp_hash availability here. That is why the following line is
+	 * commented out. */
+	/* shmem_safety_check(); */
+
+	if (pgsp != NULL && is_allowed_role())
 		pgsp->is_enabled = state;
 }
 
@@ -345,14 +335,24 @@ show_state()
 		return "off";
 }
 
-void
+static void
 set_format(const int format)
 {
-	shmem_safety_check();
-	if (is_allowed_role())
+	/* Shared memory may not be fully available at server start, so we do not
+	 * check for pgsp_hash availability here. That is why the following line is
+	 * commented out. */
+	/* shmem_safety_check(); */
+
+	if (pgsp != NULL && is_allowed_role())
 		pgsp->plan_format = format;
 }
 
+static void
+prop_format_to_shmem(int newval, void *extra)
+{
+	set_format(newval);
+}
+
 static const char *
 show_format()
 {
@@ -368,7 +368,7 @@ show_format()
 		elog(ERROR, "unexpected plan_format value: %d", pgsp->plan_format);
 }
 
-inline void
+static inline void
 shmem_safety_check(void)
 {
 	if (pgsp && pgsp_hash)
@@ -377,10 +377,10 @@ shmem_safety_check(void)
 	ereport(ERROR,
 	        (errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
 	         errmsg("pg_show_plans must be loaded"
-	                "via shared_preload_libraries")));
+	                " via shared_preload_libraries")));
 }
 
-bool
+static bool
 is_allowed_role(void)
 {
 	bool is_allowed_role = false;
@@ -394,7 +394,7 @@ is_allowed_role(void)
 }
 
 #if PG_VERSION_NUM >= 150000
-void
+static void
 pgvp_shmem_request(void)
 {
 	if (prev_shmem_request_hook)
@@ -404,7 +404,7 @@ pgvp_shmem_request(void)
 }
 #endif
 
-void
+static void
 pgsp_shmem_startup(void)
 {
 	bool    found;
@@ -443,7 +443,7 @@ pgsp_shmem_startup(void)
 	LWLockRelease(AddinShmemInitLock);
 }
 
-void
+static void
 pgsp_ExecutorStart(QueryDesc *queryDesc, int eflags)
 {
 	ExplainState *es;
@@ -473,7 +473,7 @@ pgsp_ExecutorStart(QueryDesc *queryDesc, int eflags)
 	pfree(es->str->data);
 }
 
-void
+static void
 pgsp_ExecutorRun(QueryDesc *queryDesc, ScanDirection direction,
                  uint64 count, bool execute_once)
 {
@@ -509,48 +509,6 @@ pgsp_ExecutorRun(QueryDesc *queryDesc, ScanDirection direction,
 	PG_END_TRY();
 }
 
-Datum
-pg_show_plans_enable(PG_FUNCTION_ARGS)
-{
-	set_state(true);
-	PG_RETURN_VOID();
-}
-
-Datum
-pg_show_plans_disable(PG_FUNCTION_ARGS)
-{
-	set_state(false);
-	PG_RETURN_VOID();
-}
-
-Datum
-pgsp_format_text(PG_FUNCTION_ARGS)
-{
-	set_format(EXPLAIN_FORMAT_TEXT);
-	PG_RETURN_VOID();
-}
-
-Datum
-pgsp_format_json(PG_FUNCTION_ARGS)
-{
-	set_format(EXPLAIN_FORMAT_JSON);
-	PG_RETURN_VOID();
-}
-
-Datum
-pgsp_format_yaml(PG_FUNCTION_ARGS)
-{
-	set_format(EXPLAIN_FORMAT_YAML);
-	PG_RETURN_VOID();
-}
-
-Datum
-pgsp_format_xml(PG_FUNCTION_ARGS)
-{
-	set_format(EXPLAIN_FORMAT_XML);
-	PG_RETURN_VOID();
-}
-
 Datum
 pg_show_plans(PG_FUNCTION_ARGS)
 {
@@ -637,9 +595,7 @@ pg_show_plans(PG_FUNCTION_ARGS)
 		 * count offset to get the desired plan. */
 		offset = 0;
 		for (i = 0; i < curr_nest; i++)
-			offset += pgvp_tmp_entry->plan_len[i];
-		if (offset > 0)
-			offset++;
+			offset += pgvp_tmp_entry->plan_len[i] + 1;
 
 		MemSet(nulls, 0, sizeof(nulls));
 		values[0] = Int32GetDatum(pgvp_tmp_entry->hash_key.pid);
